import uuid
from django.db import models
from django_resaas.core.base.models import TimeModel


class LayoutSetting(TimeModel):

    nome = models.CharField(max_length=100)

    # 🌗 tema
    dark_mode = models.BooleanField(default=False)

    # BOTÕES
    buttonDense = models.BooleanField(default=False)
    buttonRounded = models.BooleanField(default=False)
    buttonSquare = models.BooleanField(default=False)

    # INPUTS
    inputDense = models.BooleanField(default=False)
    inputRounded = models.BooleanField(default=False)
    inputSquare = models.BooleanField(default=False)


    # 📏 densidade UI
    density = models.CharField(
        max_length=20,
        choices=[
            ("dense", "Dense"),
            ("normal", "Normal"),
            ("comfortable", "Comfortable")
        ],
        default="normal"
    )



    # 🌊 animações
    animations = models.BooleanField(default=True)

    # ========================
    # 🔘 BUTTON
    # ========================
    button_style = models.CharField(
        max_length=20,
        choices=[
            ("flat", "Flat"),
            ("outline", "Outline"),
            ("unelevated", "Unelevated"),
            ("push", "Push")
        ],
        default="unelevated"
    )

    button_dense = models.BooleanField(default=False)
    button_round = models.BooleanField(default=False)

    # ========================
    # 📝 INPUT
    # ========================
    input_style = models.CharField(
        max_length=20,
        choices=[
            ("outlined", "Outlined"),
            ("filled", "Filled"),
            ("standout", "Standout"),
        ],
        default="outlined"
    )

    input_dense = models.BooleanField(default=False)


   # BORDER RADIUS GLOBAL
    border_radius = models.IntegerField(default=8)
    

    # ========================
    # 📂 SIDEBAR
    # ========================
    sidebar_mini = models.BooleanField(default=False)
    sidebar_width = models.IntegerField(default=260)

    # ========================
    # 📊 TOOLBAR
    # ========================
    toolbar_dense = models.BooleanField(default=False)
    toolbar_elevated = models.BooleanField(default=True)


    nome = models.CharField(max_length=100)

    # 🎨 botão
    button_style = models.CharField(
        max_length=20,
        choices=[
            ("flat", "Flat"),
            ("outline", "Outline"),
            ("unelevated", "Unelevated"),
            ("push", "Push"),
        ],
        default="unelevated"
    )

    # 📏 densidade
    dense = models.BooleanField(default=False)

    # 🔘 bordas
    rounded = models.BooleanField(default=False)
    square = models.BooleanField(default=False)

    # 🌊 ripple
    ripple = models.BooleanField(default=True)

    # 🧱 sombra
    elevated = models.BooleanField(default=True)

    # 📦 card style
    card_flat = models.BooleanField(default=False)
    card_bordered = models.BooleanField(default=False)


    # 📂 sidebar
    sidebar_mini = models.BooleanField(default=False)

    class Meta:
        verbose_name = 'LayoutSetting'
        verbose_name_plural = 'LayoutSettings'
    
    def __str__(self):
        return self.nome

    def to_dict(self):

        return {

            "dark_mode": self.dark_mode,
            "density": self.density,
            "rounded": self.rounded,
            "square": self.square,
            "animations": self.animations,

            "button": {
                "style": self.button_style,
                "dense": self.button_dense,
                "round": self.button_round,
            },

            "input": {
                "style": self.input_style,
                "dense": self.input_dense,
            },

            "card": {
                "flat": self.card_flat,
                "bordered": self.card_bordered
            },

            "sidebar": {
                "mini": self.sidebar_mini,
                "width": self.sidebar_width
            },

            "toolbar": {
                "dense": self.toolbar_dense,
                "elevated": self.toolbar_elevated
            }
        }
      


class AnimationSetting(TimeModel):

    # =========================
    # GLOBAL
    # =========================

    nome = models.CharField(max_length=100)

    enable_animations = models.BooleanField(default=True)

    animation_speed = models.CharField(
        max_length=20,
        default="normal",
        choices=[
            ("slow", "Slow"),
            ("normal", "Normal"),
            ("fast", "Fast"),
        ]
    )

    # =========================
    # PAGE TRANSITIONS
    # =========================

    page_transition = models.CharField(
        max_length=50,
        default="fade",
        choices=[
            ("none", "None"),
            ("fade", "Fade"),
            ("slide-left", "Slide Left"),
            ("slide-right", "Slide Right"),
            ("slide-up", "Slide Up"),
            ("slide-down", "Slide Down"),
            ("scale", "Scale"),
            ("zoom", "Zoom"),
        ]
    )

    # =========================
    # BUTTON EFFECTS
    # =========================

    button_animation = models.CharField(
        max_length=50,
        default="ripple",
        choices=[
            ("none", "None"),
            ("ripple", "Ripple"),
            ("pulse", "Pulse"),
            ("bounce", "Bounce"),
            ("scale", "Scale"),
        ]
    )

    # =========================
    # HOVER EFFECTS
    # =========================

    hover_effect = models.BooleanField(default=True)

    hover_style = models.CharField(
        max_length=50,
        default="lift",
        choices=[
            ("none", "None"),
            ("lift", "Lift"),
            ("shadow", "Shadow"),
            ("grow", "Grow"),
            ("glow", "Glow"),
        ]
    )

    # =========================
    # CARD ANIMATIONS
    # =========================

    card_animation = models.CharField(
        max_length=50,
        default="fade",
        choices=[
            ("none", "None"),
            ("fade", "Fade"),
            ("slide-up", "Slide Up"),
            ("zoom", "Zoom"),
        ]
    )

    # =========================
    # MODAL ANIMATION
    # =========================

    modal_animation = models.CharField(
        max_length=50,
        default="scale",
        choices=[
            ("none", "None"),
            ("scale", "Scale"),
            ("fade", "Fade"),
            ("slide-up", "Slide Up"),
        ]
    )

    def to_dict(self):
        return {
            "enable_animations": self.enable_animations,
            "animation_speed": self.animation_speed,
            "page_transition": self.page_transition,
            "button_animation": self.button_animation,
            "hover_effect": self.hover_effect,
            "hover_style": self.hover_style,
            "card_animation": self.card_animation,
            "modal_animation": self.modal_animation,
        }

    class Meta:
        verbose_name = "Animation Setting"
        verbose_name_plural = "Animation Settings"

    def __str__(self):
        return self.nome


from functools import wraps

from django.contrib.auth.models import Permission, Group

from rest_framework import status
from rest_framework.permissions import BasePermission
from rest_framework.exceptions import PermissionDenied
from rest_framework.response import Response

from django_resaas.core.utils.translate import Translate
from django_resaas.models.sucursal_user_group import SucursalUserGroup
from django_resaas.models.entidade_modulo import EntidadeModulo


class HasAppPermission(BasePermission):
    """
    Permission base do DRF integrada com o sistema multi-tenant.

    Usa o mesmo motor de permissões:
    - headers (ET, E, S, G, L)
    - grupos
    - permissões do Django
    - uma única query
    """

    message = 'Permission denied'

    def has_permission(self, request, view):
        """
        Verificação antes da execução da view.
        A view deve definir `permission_codename`.
        """

        # A view precisa declarar explicitamente a permissão
        codename = getattr(view, 'permission_codename', None)

        if not codename:
            # Segurança: se não declarou, não passa
            return False

        allowed = check_permission(
            request=request,
            role=codename
        )

        if not allowed:
            # mantém compatibilidade com DRF
            raise PermissionDenied(Translate.tdc(request,self.message))

        return True


def check_permission(request, role):
    role = role 

    if not all([
        request.user,
        request.user.is_authenticated,
        request.tipo_entidade_id,
        request.entidade_id,
        request.sucursal_id,
        request.group_id,
        request.lang_id,
    ]):
        return False

    return SucursalUserGroup.objects.filter(
        user=request.user,
        group_id=request.group_id,
        sucursal_id=request.sucursal_id,
        sucursal__entidade_id=request.entidade_id,
        sucursal__entidade__tipo_entidade_id=request.tipo_entidade_id,
        group__permissions__codename=role,
    ).exists()


def hasModulo(codigo):
    def decorator(func):
        @wraps(func)
        def wrapper(self, request, *args, **kwargs):
            entidade_id = request.entidade_id

            if not EntidadeModulo.objects.filter(
                entidade_id=entidade_id,
                modulo__codigo=codigo,
                estado=1
            ).exists():
                return Response(
                    {"detail": "Módulo não ativo"},
                    status=status.HTTP_403_FORBIDDEN
                )

            return func(self, request, *args, **kwargs)
        return wrapper
    return decorator


def hasPermission(role=None):
    def decorator(view_func):
        @wraps(view_func)
        def wrapper(self, request, *args, **kwargs):
            if check_permission(request, role):
                return view_func(self, request, *args, **kwargs)

            txt = Translate.tdc(request, 'Permission denied')
            return Response( {'alert_error': txt}, status=status.HTTP_403_FORBIDDEN )
        return wrapper
    return decorator

def isPermited( request=None, role=None):
    return check_permission(request, role)




from django.db import transaction
from django.contrib.auth.models import Group
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from rest_framework.permissions import IsAuthenticated

from django_resaas.models.tipo_entidade import TipoEntidade
from django_resaas.models.entidade import Entidade
from django_resaas.models.sucursal import Sucursal
from django_resaas.models.entidade_user import EntidadeUser
from django_resaas.models.sucursal_user import SucursalUser
from django_resaas.models.sucursal_user_group import SucursalUserGroup

from django_resaas.models.theme import Theme, Typography
from django_resaas.models.layout_setting import LayoutSetting, AnimationSetting
from django_resaas.data.theme.serializers.theme import ThemeSerializer, TypographySerializer
from django_resaas.data.layout_setting.serializers.layout_setting import LayoutSettingSerializer, AnimationSettingSerializer

from django_resaas.core.utils.translate import Translate
from django_resaas.core.utils import all

class SiteAPIView(APIView):
    def get(self, request):

        origin = request.headers.get("Origin")
        try:
            entidade = Entidade.objects.get(site=origin)
        except Exception as e:
            pass



        if entidade.theme:
            theme = Theme.objects.get(id=entidade.theme.id).to_dict()
        else:
            tipoentidade = TipoEntidade.objects.get(id=entidade.tipo_entidade.id )
            theme = Theme.objects.get(id=tipoentidade.theme.id).to_dict()

        if entidade.typography:
            typography = Typography.objects.get(id=entidade.typography.id).to_dict()
        else:
            tipoentidade = TipoEntidade.objects.get(id=entidade.tipo_entidade.id )
            typography = Typography.objects.get(id=tipoentidade.typography.id).to_dict()

        if entidade.layout_settings:
            layout_settings = LayoutSetting.objects.get(id=entidade.layout_settings.id).to_dict()
        else:
            tipoentidade = TipoEntidade.objects.get(id=entidade.tipo_entidade.id )
            layout_settings = LayoutSetting.objects.get(id=tipoentidade.layout_settings.id).to_dict()
        

        if entidade.animation_settings:
            animation_settings = LayoutSetting.objects.get(id=entidade.animation_settings.id).to_dict()
        else:
            tipoentidade = TipoEntidade.objects.get(id=entidade.tipo_entidade.id )
            animation_settings = AnimationSetting.objects.get(id=tipoentidade.animation_settings.id).to_dict()

        
       
        return all(request, layout_settings = layout_settings, theme = theme, animation_settings= animation_settings, typography= typography, bach_end=request.get_host(), entidade= entidade.id )




from rest_framework import serializers
from django_resaas.models.user import User



class ResetPasswordEmailRequestSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('id', 'email', 'username', 'password', 'mobile')
        extra_kwargs = {
            'password': {'write_only': True}
        }


import uuid

from django.db import models
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
from rest_framework_simplejwt.tokens import RefreshToken
from django_resaas.models.pessoa import Pessoa
from django_resaas.core.base.models import TimeModel


class UserManager(BaseUserManager):
    def create_user(self, username, email, password=None, mobile=None):
        user = self.model(
            username=username,
            email=self.normalize_email(email),
            mobile=mobile
        )
        user.set_password(password)
        user.save()
        return user

    def create_superuser(self, username, email, password=None):
        if password is None:
            raise TypeError('Password should not be none')

        user = self.create_user(username, email, password)
        user.is_superuser = True
        user.is_verified = True
        user.is_staff = True
        user.save()
        return user


AUTH_PROVIDERS = (
    ('email', 'email'),
    ('facebook', 'facebook'),
    ('google', 'google'),
    ('twitter', 'twitter'),
    ('mobile', 'maobile')
)


def profile_image_path(instance, file_name):
    return f'images/users/{instance.id}/{file_name}'


class User(AbstractBaseUser, PermissionsMixin, TimeModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    perfil = models.ImageField(
        default='user.png',
        upload_to=profile_image_path,
        null=True,
        blank=True
    )

    username = models.CharField(max_length=255, unique=False)
    first_name = models.CharField(max_length=255, unique=False, null=True, blank=True)
    last_name = models.CharField(max_length=255, unique=False, null=True, blank=True)

    mobile = models.CharField(
        max_length=55,
        null=True,
        unique=True,
        blank=True,
        default=None
    )
    is_verified_mobile = models.BooleanField(default=False)
    counter = models.IntegerField(default=0)
    email = models.EmailField(
        max_length=255,
        null=True,
        unique=True,
        blank=True,
        default=None
    )

    language = models.ForeignKey(
        "django_resaas.Idioma",
        on_delete=models.SET_NULL,
        null=True,
        blank=True
    )



    is_verified_email = models.BooleanField(default=False)
    is_active = models.BooleanField(default=True)
    is_staff = models.BooleanField(default=False)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    auth_provider = models.CharField(
        max_length=255,
        default='email',
        choices=AUTH_PROVIDERS
    )

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['username']

    objects = UserManager()

    def save(self, *args, **kwargs):
        if self.email == '':
            self.email = None
        if self.mobile == '':
            self.mobile = None
        super().save(*args, **kwargs)

    class Meta:
        verbose_name = "User"
        verbose_name_plural = "Users"
        permissions = ()

    def __str__(self):
        return self.username

    def tokens(self):
        refresh = RefreshToken.for_user(self)
        return {
            'refresh': str(refresh),
            'access': str(refresh.access_token),
        }






from django_resaas.core.base.serializers import BaseSerializer
from django_resaas.models.user import User
from rest_framework import serializers
from django_resaas.models.idioma import Idioma


class MeSerializer(BaseSerializer):
    language = serializers.SerializerMethodField()

    def get_language(self, obj):
        if not obj.language:
            idioma = Idioma.objects.filter(code="pt-pt").first()
            return {
                "id": idioma.id,
                "nome": idioma.nome,
                "code": idioma.code,
            }

        return {
            "id": obj.language.id,
            "nome": obj.language.nome,
            "code": obj.language.code,
        }

    class Meta:
        model = User
        fields = ['id', 'email', 'username', 'perfil', 'mobile', 'language', 'last_login']
